<?php

/* Layout */

class Layout {
  
  var $classes;
  var $callbacks;
  var $column_widths;
  var $open_container = "\n<section class=\"block container\">\n";
  var $close_container = "\n</section><!-- .block -->\n";
  
  function __construct() { global $der, $layout_callbacks;
    
    // Set layout callbacks
    require(TEMPLATEPATH . '/includes/layout/callbacks.php');
    
    $this->callbacks = $layout_callbacks;
    
    // Set columns widths
    $this->column_widths = array(
      1 => 40,
      2 => 100,
      3 => 160,
      4 => 220,
      5 => 280,
      6 => 340,
      7 => 400,
      8 => 460,
      9 => 520,
      10 => 580,
      11 => 640,
      12 => 700,
      13 => 760,
      14 => 820,
      15 => 880,
      16 => 940,
      'one-third' => 300
    );
    
    // Set classes
    $this->classes = array(
      1 => 'one column',
      2 => 'two columns',
      3 => 'three columns',
      4 => 'four columns',
      5 => 'five columns',
      6 => 'six columns',
      7 => 'seven columns',
      8 => 'eight columns',
      9 => 'nine columns',
      10 => 'ten columns',
      11 => 'eleven columns',
      12 => 'twelve columns',
      13 => 'thirteen columns',
      14 => 'fourteen columns',
      15 => 'fifteen columns',
      16 => 'sixteen columns'
    );
    
  }
  
  /* Renders a nested container */
  
  function render_nested_container($container, $parent_columns) { global $content_width;

    $slot = $container[0];
    
    // pre($slot);

    if ($slot['s'] == 16 && is_array($slot['c']) && $this->callbacks[$slot['c']['s']][1] == 16) {

      return;
      
    } else {

      // Normal component(s)
      
      foreach ($container as $slot) {
        
        if (is_array($slot['c'])) {
          
          //////////////// CONTENT WIDTH
          $content_width = $this->column_widths[$slot['s']];
          
          // Component available
          $comp = $slot['c'];
          $options = $comp['o'];
          $options['slots'] = $parent_columns;
          $options['columns_class'] = $this->column_class($parent_columns) . ' alpha omega';
          
          $callback = $this->callbacks[$comp['s']][0];
          echo $callback($options, (isset($options['content']) ? $options['content'] : null), $comp['s']) . "\n\n";
          
        } else {
          
          // Empty slot
          
          $this->empty_slot($slot);
          
        }
      }

    }

  }
  
  /* Renders a container */
  
  function render_container($container) { global $content_width;

    $slot = $container[0];
    
    // pre($slot);

    if ($slot['s'] == 16 && is_array($slot['c']) && $this->callbacks[$slot['c']['s']][1] == 16) {

      // Full width component
      
      //////////////// CONTENT WIDTH
      $content_width = $this->column_widths[16];
  
      $comp = $slot['c'];
      $options = $comp['o'];
      $options['slots'] = $slot['s'];
      $options['columns_class'] = $this->column_class($slot['s']);
      
      echo "\n";
      
      $callback = $this->callbacks[$comp['s']][0];
      echo $callback($options, (isset($options['content']) ? $options['content'] : null), $comp['s']) . "\n";
    
      echo "\n";
      
    } else {

      // Normal component(s)
      
      echo $this->open_container;

      foreach ($container as $slot) {
        
        if (is_array($slot['c'])) {
          
          //////////////// CONTENT WIDTH
          $content_width = $this->column_widths[$slot['s']];
          
          // Component available
          $comp = $slot['c'];
          $options = $comp['o'];
          $options['slots'] = $slot['s'];
          $options['columns_class'] = $this->column_class($slot['s']);
          
          $callback = $this->callbacks[$comp['s']][0];
          echo $callback($options, (isset($options['content']) ? $options['content'] : null), $comp['s']) . "\n\n";
          
        } else {
          
          // Empty slot
          
          $this->empty_slot($slot);
          
        }
      }

      echo $this->close_container;

    }

  }
    
  /* Gets the column class */
  
  function column_class($cols) {
    switch ($cols) {
      case 'one-third':
      case 16/3.0:
        return 'one-third column';
      default:
        return $this->classes[$cols];
    }
  }
  
  /* Renders an empty slot */
  
  function empty_slot($slot) {
    printf("\n\n".'<div class="%s"></div>'."\n", $this->column_class($slot['s']));
  }
  
  /* Generates code for a shortcode call */
  
  function gen_shortcode($shortcode, $options) {
    $code = '';
    
    // Opening
    $code = sprintf('[%s', $shortcode);
    
    foreach ($options as $key => $val) {
      if ($key == 'content') continue;
      $val = preg_replace('/"/', '\\"', $val); // Escape double quotes
      $code .= sprintf(' %s="%s"', $key, $val);
    }
    
    if (array_key_exists('content', $options)) {
      // Long form
      $code .= sprintf(']%s[/%s]', $options['content'], $shortcode);
    } else {
      // Short form
      $code .= ' /]';
    }
    
    return $code;
  }
  
  /* Parses options */
  
  function parse_options($options) {
    $out = array();
    foreach ($options as $option) {
      switch ($option->type) {
        case 'text':
        case 'textarea':
          $out[$option->id] = ($option->value) ? $option->value : null;
          break;
        case 'select':
        case 'radio':
          foreach ($option->values as $o) {
            if ( (property_exists($o, 'selected') && $o->selected) || (property_exists($o, 'checked') && $o->checked) ) {
              $out[$option->id] = $o->value;
              break;
            }
          }
          break;
        case 'checkbox':
          $checked = array();
          foreach ($option->values as $o) {
            if ( property_exists($o, 'checked') && $o->checked ) {
              $checked[] = $o->value;
            }
          }
          $out[$option->id] = implode(', ', $checked);
          break;
        default:
          break;
      }
    }
   return $out; 
  }
  
}

?>