<?php

$theme_sidebars = array();
$theme_admin_menus = array();
$theme_options = array();

class Theme {

  var $metabox;
  var $prefix;
  var $locales;
  var $theme_data;
  var $mustache;
  var $home_url;
  
  var $layout;
  var $layouts;
  var $layout_keys;
  var $layout_to_render;
  var $options;
  var $default_options;
  var $options_context = 'theme-options';
  
  var $last_query = null;
  var $tabs_context = null;
  var $rendering_layout = false;
  var $load_maps_api = false;
  
  private $pagination_defaults;
  private $widget_defaults;
  private $template_directory_uri;
  private $more_tag = '<!--more-->';
  private $mustache_regex = '/^Mustache_/';
  private $mustache_path;

  ///////////////////////////////////////////////////////
  // CONSTRUCTOR
  ///////////////////////////////////////////////////////
  
  function __construct() {
    
    // Set theme prefix
    $this->prefix = sprintf('_%s_', THEME_ID);
    
    // Cache home url
    $this->home_url = home_url();

    // Initialize pagination options
    $this->pagination_defaults = array(
      'first' => '&lsaquo;',
      'last' => '&rsaquo;',
      'prev' => '&laquo; Prev',
      'next' => 'Next &raquo;',
      'page_class' => 'page',
      'active_class' => 'active',
      'nav_class' => 'nav',
      'prev_class' => 'prev',
      'next_class' => 'next',
      'nav' => true,
      'nav_jump' => true,
      'range' => 2
    );
    
    // Initialize widget defaults
    $this->widget_defaults = array(
      'name' => null,
      'id' => null,
      'description' => '',
      'before_widget' => "\n" . '<div class="widget %2$s" id="%1$s">' . "\n",
      'after_widget' => "\n</div><!-- .widget -->\n",
      'before_title' => '<h3 class="title">',
      'after_title' => '</h3>' . "\n"
    );
    
    // Template directory URI
    $this->template_directory_uri = get_template_directory_uri();

    // Theme data
    $this->theme_data = wp_get_theme();
    
    // Theme version
    if (!defined('THEME_VERSION')) define('THEME_VERSION', $this->theme_data->get('Version'));
    
    // Theme Options. Loaded as needed
    $this->options = array();
    
    // Default theme options. Loaded as needed
    $this->default_options = array();
    
    // Set layouts
    $this->layouts = $this->get_option_array('layouts');
    
    // Set layout keys
    $this->layout_keys = implode(', ', array_keys($this->layouts));

    // Set layout instance
    require('class.layout.php');
    $this->layout = new Layout();
    
    // Register mustache loader
    $this->mustache_path = TEMPLATEPATH . '/framework/mustache';
    spl_autoload_register(array($this, 'mustache_autoload'));
    
    // Load mustache instance
    $this->mustache = new Mustache_Engine(array(
      'cache' => TEMPLATEPATH . '/cache/mustache',
      'charset' => 'UTF-8',
      'escape' => 'esc_html'
    ));
    
  }
  
  ///////////////////////////////////////////////////////
  // METHODS
  ///////////////////////////////////////////////////////
  
  /* Render nested layout */
  
  function render_nested_layout($layout, $args) {
    
    printf("\n".'<div class="component %s nested-layout clearfix">'."\n\n<!-- [NESTED LAYOUT] -->\n\n", $args['columns_class']);
    
    $data = $this->layouts[$layout];
    
    foreach ($data as $container) {
      
      $this->layout->render_nested_container($container, $args['slots']);
      
    }
    
    echo "<!-- [/NESTED_LAYOUT] -->\n\n</div>";
    
  }
  
  /* Mustache Autoloader */
  
  function mustache_autoload($class) {
    if (preg_match($this->mustache_regex, $class)) {
      $path = str_replace('Mustache/', $this->mustache_path . '/', str_replace('_', '/', $class)) . '.php';
      if (is_file($path)) require($path);
    }
  }
  
  /* Checks if a post has a layout assigned */
  
  function has_layout($id) {
    $layout = ( is_home() || is_front_page() || is_search() || is_archive() || is_404() ) ? null : $this->postmeta('layout');
    if (empty($layout) || !$this->layout_exists($layout)) $layout = $this->option($id);
    $layout_exists = ($layout && $this->layout_exists($layout));
    if ($layout_exists) $this->layout_to_render = $layout;
    return $layout_exists;
  }
  
  /* Renders the layout for a post or page */
  
  function render_layout() {
    $layout = (!empty($this->layout_to_render)) ? $this->layout_to_render : $this->postmeta('layout');
    $this->layout_to_render = null;
    $this->layout($layout);
  }
  
  /* Renders a message in a section */
  
  function message($str) {
    $this->open_section();
    echo '<div class="sixteen columns">'."\n";
    echo apply_filters('the_excerpt', $str);
    echo '</div>'."\n";
    $this->close_section();
  }
  
  /* Renders the metadata display */
  
  function metadata_display($setting) {
    switch ($setting) {
      case 'description':
        return esc_html($this->postmeta('description'));
      case 'author':
        return $this->get_post_metadata('author');
      case 'author-date':
        return $this->get_post_metadata('author-date');
      case 'date':
        return $this->get_post_metadata('date');
      case 'raw-date':
        return $this->get_post_metadata('raw-date');
      case 'cats':
        return $this->get_post_metadata('category');
      case 'tags':
        return $this->get_post_metadata('tags');
      case 'cats-tags':
        return $this->get_post_metadata('category, tags');
        break;
      case 'author-date-cats':
        return $this->get_post_metadata('author-date, category');
      case 'author-date-cats-tags':
        return $this->get_post_metadata('author-date, category, tags');
      case 'author-cats':
        return $this->get_post_metadata('author, category');
      case 'author-cats-tags':
        return $this->get_post_metadata('author, category, tags');
      case 'query-details':
        if (is_category()) {
          return __("Category Archive", "theme");
        } elseif (is_tag()) {
          return __("Tag Archive", "theme");
        } else {
          return null;
        }
        break;
    }
  }
  
  /* Returns post metadata */
  
  function get_post_metadata($keys) {
    $out = array();
    $post = get_post(get_the_ID());
    $post_type = $post->post_type;
    $keys = csv2array($keys);
    foreach ($keys as $key) {
      switch ($key) {
        case 'author':
        case 'author-date':
          $author = sprintf('<a rel="author" href="%s">%s</a>', 
            get_author_posts_url($post->post_author),
            get_the_author_meta('display_name', $post->post_author)
          );
          if ($key == 'author-date') {
            $out[] = sprintf(__('By %s on %s.', "theme"), $author, get_the_date());
          } else if ($key == 'author') {
            $out[] = sprintf(__('By %s.', "theme"), $author);
          }
          break;
        case 'category':
          // Assumed to be a taxonomy
          if ($post_type == 'post') $taxonomy = 'category';
          else if ($post_type == 'portfolio') $taxonomy = 'portfolio-category';
          else $taxonomy = null;
          if ($taxonomy) {
            $out[] = get_the_term_list($post->ID, $taxonomy, sprintf("%s ", __("Posted in", "theme")), ', ', '.');
          }
          break;
        case 'date':
          $date = sprintf(__('Published on %s.', "theme"), get_the_date());
          $out[] = $date;
          break;
        case 'raw-date':
          $out[] = get_the_date();
          break;
        case 'tags':
          if ($post_type == 'post') $taxonomy = 'post_tag';
          else $taxonomy = null;
          if ($taxonomy) {
            $terms = get_the_term_list($post->ID, $taxonomy, sprintf("%s ", __("Tagged as", "theme")), ', ', '.');
            if ($terms) {
              $out[] = $terms;
            }
          }
          break;
      }
    }
    return implode(' ', $out);
  }
  
  /* Renders a mustache template */
  
  function render($template, $vars) {
    return $this->mustache->render($template, $vars);
  }
  
  function layout($key) {
    $this->rendering_layout = true;
    $layout = $this->layouts[$key];
    foreach ($layout as $container) {
      $this->layout->render_container($container);
    }
    $this->rendering_layout = false;
  }
  
  /* Renders a layout for a specific option */
  
  function layout_for_option($id, $config_url=null) {
    
    $layout = $this->option($id);
    
    if ( $this->layout_exists($layout) ) {

      $this->layout($layout);
      
    } else if ($config_url && is_user_logged_in()) {
      
      echo '
<div style="width: 940px; margin: 0 auto; padding: 30px 0;">
' . __("This page has no layout assigned.", "theme") . '&nbsp; <a target="_blank" href="' . $config_url . '">' . __("Add one", "theme") . '</a>.
</div>';
    }

  }
  
  /* Checks if a layout exists */
  
  function layout_exists($layout) {
    return array_key_exists($layout, $this->layouts);
  }
  
  /* Opens a section */
  
  function open_section() {
    echo $this->layout->open_container;
  }
  
  /* Adds a separator between containers */
  
  function middle() {
    $this->close_section();
    $this->open_section();
  }
  
  /* Close session */
  
  function close_section() {
    echo $this->layout->close_container;
  }
  
  /* Returns a theme option */
  
  function option($id, $default=null) {
    $options = array_key_exists($this->options_context, $this->options) ? $this->options[$this->options_context] : null;
    if (is_array($options)) {
      $out = isset($options[$id]) ? $options[$id] : null;
      if (empty($out)) {
        
        if ($default) {
          return $default;
        } else {
          
          if (isset($this->default_options[$this->options_context][$id])) {
            return $this->default_options[$this->options_context][$id];
          } else {
            return null;
          }

        }
        
      } else {
        return (is_string($out)) ? stripslashes($out) : $out;
      }
    } else {
      // Retrieve options for context
      $this->load_options($this->options_context);
      
      // Recursively run the method, this time with the options set
      return $this->option($id, $default);
    }
  }
  
  /* Returns multiple options */
  
  function options($str) {
    $options = csv2array($str);
    $out = array();
    foreach ($options as $key) {
      $out[$key] = $this->option($key);
    }
    return $out;
  }
  
  /* Returns a theme option, making sure it's a boolean */
  
  function option_bool($id, $default=null) {
    $out = $this->option($id, $default);
    if (is_string($out)) {
      $out = strtolower($out);
      return ($out == 'yes' || $out == 'enabled');
    } else {
      return false;
    }
  }
  
  private function load_options($context) { global $theme_options;
    
    if ( ! is_admin() ) {
      // Backup current theme options (if any)
      $backup_theme_options = $theme_options;
      $theme_options = array();

      // Retrieve defaults for context
      $defaults = array();
      $path = $this->path(sprintf('includes/options/%s.php', $context));
      
      include_once($path);

      foreach ($theme_options as $section => $options) {
        foreach ($options as $title => $args) {
          if (isset($args['id']) && isset($args['default'])) {
            $defaults[$args['id']] = $args['default'];
          }
        }
      }
      
      $this->default_options[$context] = $defaults;
      
      // Restore current theme options
      $theme_options = $backup_theme_options;
    }
    
    ////////////// DO NOT DELETE
    
    // Retrieve options for context
    $this->options[$this->options_context] = $this->get_option_array($this->options_context);
    
    ////////////// DO NOT DELETE
    
  }
  
  /* Sets an option context */
  
  function set_context($context) {
    $this->options_context = $context;
  }
  
  /* Sets the default option context */
  
  function default_context() {
    $this->set_context('theme-options');
  }
  
  /* Resets the theme options context */
  
  function reset_context() {
    $this->default_context();
  }
  
  /* Returns the context key, used to save multiple options */
  
  function context_key() {
    return $this->key($this->options_context);
  }
 
  /* Returns a WordPress option */
  
  function get_option($id) {
    return get_option($this->key($id));
  }
  
  /* Returns an option, ensuring an array return type */
  
  function get_option_array($id) {
    $opt = $this->get_option($id);
    if (empty($opt)) return array();
    else if (is_array($opt)) return $opt;
    else return unserialize($opt);
  }
  
  /* Updates a worpdress WordPress option */
  
  function update_option($id, $val) {
    return update_option($this->key($id), $val);
  }
  
  /* Removes a WordPress option */
  
  function delete_option($id) {
    return delete_option($this->key($id));
  }
  
  /* Returns a key prefixed with the theme id */
  
  function key($key) {
    return $this->prefix . strtolower($key);
  }
  
  /* Returns a theme uri */
  
  function uri($path='', $ver='') {
    if ($ver) $ver = (preg_match('/\?/', $path) ? '&' : '?') . "ver=${ver}";
    return $this->template_directory_uri . '/' . $path . $ver;
  }
  
  /* Returns a theme path */
  
  function path($path) {
    return TEMPLATEPATH . '/' . $path;
  }
  
  /* Enables option pages */

  function option_pages($arr) { global $theme_admin_menus;
    $theme_admin_menus = $arr;
  }
    
  /* Theme Nonce */
  
  function nonce() {
    wp_nonce_field(THEME_NONCE_VERIFY, THEME_NONCE); echo "\n";
  }
  
  /* Checks if a nonce is valid */
  
  function verify_nonce() {
    $method = $_SERVER['REQUEST_METHOD'];
    if ($method == 'GET') {
      return ( isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce']) );
    } else if ($method == 'POST') {
      return ( isset($_POST[THEME_NONCE]) AND wp_verify_nonce($_POST[THEME_NONCE], THEME_NONCE_VERIFY) );
    }
  }
  
  /* 
    Returns a meta option from the active post in the loop.
    This method takes into account the meta prefix.
  */
  
  function postmeta($key, $multiple=false) {
    
    $post_id = get_the_ID();

    return $this->get_postmeta($post_id, $key, $multiple);
    
  }
  
  /* Returns a prefixed meta key */
  
  /* 
    Wrapper for get_post_meta. Returns a meta key from a Post ID. 
    This method takes into account the meta prefix.
  */
  
  function get_postmeta($post_id, $key, $multiple=false) {
    
    // Process multiple values (checkboxes)
    if ($multiple) {
      
      $out = array();
      $data = get_post_custom($post_id);
      $regex = sprintf("/^%s:%s:_/", $this->prefix, $key);
      
      foreach ($data as $k => $v) {
        if (preg_match($regex, $k)) {
          $val = preg_replace($regex, '', $k);
          $out[] = (is_numeric($val)) ? (int) $val : stripslashes($val);
        }
      }
      
      return $out;
      
    // Process regular postmeta key
    } else {
      
      $key = $this->key($key);
      
      return stripslashes(get_post_meta($post_id, $key, true));
      
    }

  }
  
  /* Loads a theme module from 'includes/modules' */
  
  function load_module($module) {
    $file = $this->path("includes/modules/${module}.php");
    include($file);
  }
  
  /* Loads various theme modules from 'include/modules' */
  
  function load_modules($args) {
    $modules = csv2array($args);
    foreach ($modules as $module) {
      $this->load_module($module);
    }
  }
  
  /* Loads a widget */
  
  function load_widget($widget) {
    $file = $this->path("includes/widgets/${widget}.php");
    include($file);
  }
  
  /* Loads theme widgets */
  
  function load_widgets($args) {
    $widgets = csv2array($args);
    foreach ($widgets as $widget) {
      $this->load_widget($widget);
    }
  }
  
  /* Loads theme metaboxes from 'includes/metaboxes' */
  
  function load_metaboxes($str) {
    $args = csv2array($str);
    foreach ($args as $file) {
      include(TEMPLATEPATH . "/includes/metaboxes/${file}.php");
    }
  }
  
  /* Gets taxonomies */
  
  function get_taxonomies($taxonomy, $slug=true) {
    $taxes = get_terms($taxonomy, array('hide_empty'=> false));
    $values = array();
    foreach ($taxes as $tax) {
      $values[ ($slug) ? $tax->slug : $tax->term_id ] = esc_html($tax->name);
    }
    return $values;
  }
  
  /* Retrieves the featured image */
  
  function post_image() {
    $post_id = get_the_ID();
    $thumb_id = get_post_thumbnail_id($post_id);
    $data = wp_get_attachment_image_src($thumb_id, '', false);
    if ($data) {
      $img = $data[0];
      return $img;
    } else {
      return null;
    }
  }
  
  /* Generates a thumbnail URL */
  
  function thumb_src($img, $w, $h=0, $align='') {
    
    // Process align query var
    if (preg_match('/^[^\?]+\?a=[a-z]{1,2}$/', $img)) {
      $img = explode('?a=', $img);
      list($img, $align) = $img;
    }
    
    return mr_image_resize($img, $w, $h, true, $align, false);

  }
  
  /* Generates a featured image thumb */
  
  function post_thumb($w, $h=0) {
    $img = $this->post_image();
    if ($img) {
      $align = $this->postmeta(THUMB_CROP_META);
      return $this->thumb_src($img, $w, $h, $align);
    } else {
      return null;
    }
  }
  
  /* Retrieves a list of files from a theme directory */
  
  function get_files($dir, $ext='', $fullpath=false) {
    $path = TEMPLATEPATH . '/' . $dir;
    $out = array();
    $files = scandir($path);
    foreach ($files as $file) {
      if ( $file == '.' OR $file == '..' OR $file[0] == '.' ) continue;
      if ( !empty($ext) ) {
        $ext = preg_replace('/\\./', '\\.', $ext);
        $regex = sprintf('/\.%s$/', $ext);
        if (preg_match($regex, $file)) {
          $out[] = ($fullpath) ? $path . '/' . $file : $file;
        }
      } else {
        $out[] = ($fullpath) ? $path . '/' . $file : $file;
      }
    }
    return $out;
  }
  
  /* Gets the site's domain */
  
  function get_hostname() {
    $parts = parse_url($this->home_url);
    return $parts['host'];
  }
  
  /* Sets pagination defaults */
  
  function set_pagination_defaults($args) {
    $this->pagination_defaults = wp_parse_args($args, $this->pagination_defaults);
  }
  
  /* Paginates a query.*/
  
  function paginate($query=null, $echo=false) { //global $paged;
    
    // Uses code from http://www.kriesi.at/archives/how-to-build-a-wordpress-post-pagination-without-plugin
    
    if (empty($query)) $query = $GLOBALS['wp_query'];
    
    // Set pagination options
    extract($this->pagination_defaults);
    
    $html = '';
    $showitems = ($range * 2) + 1;

    $paged = defined('__PAGED__') ? __PAGED__ : get_query_var('paged');

    if (empty($paged)) $paged = 1;

    $pages = $query->max_num_pages;
    
    if ($pages != -1) {
      
      if ($nav_jump && $paged > 2 && $paged > $range+1 && $showitems < $pages) $html .= "\n" . '  <li class="nav rewind"><a href="' . get_pagenum_link(1) . '">' . $first . '</a></li>';
      if ($nav && $paged > 1 && $showitems < $pages) $html .= "\n" . '  <li><a href="' . get_pagenum_link($paged - 1) . '">' . $prev . '</a></li>';

      for ($i=1; $i <= $pages; $i++) {
        if ($pages != -1 && ( ! ($i >= $paged+$range+1 || $i <= $paged-$range-1) || $pages <= $showitems )) {
          
          if ($paged == $i) $active_page = $i;
          
          $html .= ($paged == $i) 
          ? "\n" . '  <li class="' . sprintf('%s %s', $active_class, $page_class) . '">' . $i . '</li>'
          : "\n" . '  <li class="' . $page_class . '"><a href="' . get_pagenum_link($i) . '">' . $i .'</a></li>';
        }
      }

      if ($nav && $paged < $pages && $showitems < $pages) $html .= "\n" . '  <li class="' . sprintf('%s %s', $nav_class, $prev_class) . '"><a href="' . get_pagenum_link($paged + 1) . '">' . $next . '</a></li>';
      if ($nav_jump && $paged < $pages-1 &&  $paged+$range-1 < $pages && $showitems < $pages) $html .= "\n" . '  <li class="nav fast-forward"><a href="' . get_pagenum_link($pages) . '">' . $last . '</a></li>';
      
      $html .= "\n</ul><!-- .pagination -->";
    }
    
    $html = '<ul class="pagination">' . sprintf(sprintf('<li class="label">%s</li>', __("Page %d of %d", "theme")), (isset($active_page) ? $active_page : ''), $pages) . $html;
    
    if ($echo) echo $html;
    else return $html;

  }

  /* Loads post type definitions from 'includes/post_types/' */
  
  function load_post_types($str) {
    $args = csv2array($str);
    foreach ($args as $type) {
      $file = $this->path("includes/post_types/${type}.php");
      include($file);
    }
  }
  
  /* Registers a custom post type. Serves as a wrapper for 'register_post_type' */
  
  function add_post_type($opts) {

    // Set $args array if not provided
    $opts = wp_parse_args($opts, array(
      'slug' => null,
      'singular_name' => null,
      'plural_name' => null,
      'args' => array()
    ));
    
    // Provides: $slug, $singular_name, $plural_name, $args
    extract($opts);
    
  	$defaults = array(
  		'public' => true,
  		'menu_position' => 50,
  		'supports' => 'title, editor, excerpt, trackbacks, comments, revisions'
  	);

  	$args = wp_parse_args($args, $defaults);

    // Convert 'suppports' string to an array
  	$args['supports'] = csv2array($args['supports']);
  	
  	$labels = isset($args['labels']) ? $args['labels'] : null;
  	
  	// Set labels
  	$args['labels'] = array(
			'name'	=>	$plural_name,
			'singular_name'	=>	$singular_name,
			'add_new'	=>	'Add New',
			'add_new_item' => 'Add New' . " ${singular_name}",
			'edit_item' => 'Edit' . " ${singular_name}",
			'new_item' => 'New' . " ${singular_name}",
			'view_item' => 'View' . " ${singular_name}",
			'search_items' => 'Search' . " ${plural_name}",
			'not_found' => sprintf('No %s found', $plural_name),
			'not_found_in_trash' => sprintf('No %s found in Trash', $plural_name),
		);

		$args['labels'] = wp_parse_args($labels, $args['labels']);
		
  	register_post_type($slug, $args);
  	
  	// If there's a column callback, run it
  	if (!empty($columns_callback) && function_exists($columns_callback)) {
  	  $this->manage_columns($slug, $columns_callback, (isset($args['hierarchical']) ? $args['hierarchical'] : null) );
  	}

  }
  
  /* Manages Post/Page Columns */
  
  function manage_columns($post_type, $callback, $hierarchical=false) {
    if (is_admin()) {
      $context = ($hierarchical) ? 'pages' : 'posts';
      add_action("manage_${context}_custom_column", $callback, 10, 2);
      add_filter("manage_{$post_type}_posts_columns", $callback);
    }
  }

  /* Registers a taxonomy for an object type */
  
  function add_taxonomy($opts) {
    
    // Set default values array if not provided
    $opts = wp_parse_args($opts, array(
      'slug' => null,
      'post_type' => null,
      'singular_name' => null,
      'plural_name' => null,
      'args' => array(),
    ));
    
    extract($opts);
    
    $defaults = array(
      'labels' => array(
  			'name'	=>	$plural_name,
  			'singular_name' => $singular_name,
  			'search_items' => 'Search' . ' ' . $plural_name, 
  			'all_items' => 'All' . ' ' . $plural_name,
  			'parent_item' => 'Parent' . ' ' . $singular_name,
  			'parent_item_colon' => 'Parent' . ' ' . $singular_name . ':',
  			'edit item' => 'Edit' . ' ' . $singular_name,
  			'update_item' => 'Update' . ' ' . $singular_name,
  			'add_new_item' => 'Add New' . ' ' . $singular_name,
  			'new_item_name' => 'New' . ' ' . (isset($post_type_name) ? $post_type_name : null) . $singular_name . ' ' . 'Name'
      )
    );
    
    $args = wp_parse_args($args, $defaults);
    
    register_taxonomy($slug, $post_type, $args); // If $post_type is null, it will be ignored
    
  }

  /* Adds Sidebars */
  
  function add_sidebar($args) { global $theme_sidebars;
    
    // Calling register sidebars multiple times is more effective than calling 'register_sidebars()'
    // http://codex.wordpress.org/Function_Reference/register_sidebar#Notes
    
    $args = wp_parse_args($args, $this->widget_defaults);
    
    // Set class default to be the same id
    if (empty($args['class']) && isset($args['id']))  $args['class'] = $args['id'];
    
    // Enqueue sidebar definition (sidebars are registered on the 'widgets_init' event)
    $theme_sidebars[] = $args;
    
  }
  
  /* Sets widget defaults, to be used by '$der->add_sidebar' */
  
  function set_widget_defaults($args) {
    $this->widget_defaults = wp_parse_args($args, $this->widget_defaults);
  }
  
  /* Renders widgets from specific sidebar id's */
  
  function widgets() {
    $args = func_get_args();
    $len = count($args);
    $limit = $len - 1;
    
    for ($i=0; $i < $len; $i++) {
      $sidebar = $args[$i];
      if ($i==0) echo "\n<!-- [${sidebar}] -->\n";
      dynamic_sidebar($sidebar);
      if ($i < $limit) {
        $next_sidebar = $args[$i+1];
        echo "\n<!-- [/${sidebar}] | [${next_sidebar}] -->\n";
      } else {
        echo "\n<!-- [/${sidebar}] -->\n";
      }
    }

    echo "\n"; // prettify
  }
  
  /* Renders a navigation menu */
  
  function menu($location) {
    
    $args = array(
      'theme_location' => $location,
      'container' => false,
      'items_wrap' => '%3$s'
    );
    
    $args['echo'] = false;
    
    $out = wp_nav_menu($args);
    
    $out = preg_replace('/^<div class="menu"><ul>|<\\/ul><\\/div>$/', '', $out);
    $out = str_replace('current_page_item', 'current_page_item current-menu-item', $out);
    
    return $out;

  }
  
  /* Better excerpt formatting */
  
  function get_excerpt($more='') {
    $post = get_post(get_the_ID());
    $more_tag = preg_match($this->more_tag, $post->post_content);
    if ($more_tag) {
      $content = explode($this->more_tag, $post->post_content);
      $content = $this->content($content[0]); // Only the teaser
    	return $content;
    } else {
      return apply_filters('the_excerpt', get_the_excerpt());
    }
  }
  
  /* Prepare content */
  
  function content($content, $no_shortcodes=false) {
    if ($no_shortcodes) $content = strip_shortcodes($content);
    $content = apply_filters('the_content', $content);
    $content = str_replace(']]>', ']]&gt;', $content);
  	return $content;
  }
  
  /* Encrypts a string securely */
  
  function encrypt($string) {
    // Note: AUTH_SALT is unique for each wp installation
    $salt = md5(AUTH_SALT);
    $mcrypt_iv = md5(sha1($salt));
    $encrypted = base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $salt, $string, MCRYPT_MODE_CBC, $mcrypt_iv));
    return $encrypted;
  }
  
  /* Encrypts an array */
  
  function encrypt_array($arr) {
    return $this->encrypt(serialize($arr));
  }
  
  /* Decrypts a string securely */
  
  function decrypt($hash) {
    // Note: AUTH_SALT is unique for each wp installation
    $salt = md5(AUTH_SALT);
    $mcrypt_iv = md5(sha1($salt));
    $decrypted = rtrim(mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $salt, base64_decode($hash), MCRYPT_MODE_CBC, $mcrypt_iv), "\0");
    return $decrypted;
  }
  
  /* Decrypts an array */
  
  function decrypt_array($hash) {
    return unserialize($this->decrypt($hash));
  }
  
  /* Loads a shortcode from 'includes/shortcodes' */
  
  function load_shortcode($shortcode) {
    $file = $this->path("includes/shortcodes/${shortcode}.php");
    include($file);
  }
  
  /* Loads shortcodes from 'includes/shortcodes' */
  
  function load_shortcodes($str) {
    $args = csv2array($str);
    foreach ($args as $shortcode) {
      $this->load_shortcode($shortcode);
    }
  }
  
}

?>
