<?php

  if (!defined('ABSPATH')) die();

  global $pagenow;
  
  /* Core Admin Functions */
  
  // Core actions
  add_action('admin_menu', 'theme_register_options_page');
  add_action('after_switch_theme', 'theme_activation_hook');
  add_action('admin_notices', 'detect_permission_issues');
  
  // Admin calls
  add_action('wp_ajax_sendmsg_submit', 'sendmsg_submit');
  add_action('wp_ajax_nopriv_sendmsg_submit', 'sendmsg_submit');
  add_action('wp_ajax_theme_options_update', 'theme_options_update_callback');
  add_action('wp_ajax_webfonts_update', 'webfonts_update_callback');
  
  // Theme admin interface assets
  add_action('admin_print_styles', 'theme_admin_stylesheet');
  add_action('admin_print_scripts', 'theme_admin_print_scripts');
  add_action('admin_head', 'theme_admin_head');
  
  // Allow thickbox uploader to show different labels
  if ($pagenow == 'media-upload.php') {
    if (!isset($_REQUEST['post_id'])) $_REQUEST['post_id'] = 0;
    add_filter('gettext', 'theme_thickbox_labels');
  }
  
  /* Detect permission issues */
  function detect_permission_issues() { global $der;
    $failed = array();
    $effects = array();
    $paths = array(
      'cache/mustache',
      'core/styles',
      'core/styles/custom.css',
    );
    foreach ($paths as $path) {
      $p = $der->path($path);
      if (!is_writable($p)) {
        $failed[] = $path;
        switch ($path) {
          case 'cache/mustache':
            $effects[] = "- Nothing will display (except header/footer), since the templates can't be written to disk.";
            break;
          case 'cache/timthumb':
            $effects[] = "- You will not see any Thumbnails, since these won't be able to be cached to disk.";
            break;
          case 'core/styles':
            $effects[] = "- You will not be able to save the changes in the Color Editor, since the stylesheet can't be saved to disk.";
            break;
          case 'core/styles/custom.css':
            $effects[] = "- The theme won't be able to write the Color Editor stylesheet, since the custom.css file is not writable.";
            break;
        }
      }
    }
    
    if (!empty($failed)) {
      
      printf('<div class="error" style="padding: 5px 5px;">
<h3 style="margin: 0.4em 0 0 0;">Permission Issues</h3>
<p>I need the following paths to be <strong>writable</strong>. Currently, they are not:</p>
<pre><strong>%s</strong></pre>
<p>If these files are not writable, I <strong>will not</strong> function properly. <strong>This is NOT a bug</strong>.</p>
<p style="padding: 10px; border: solid 1px #E6DB55; background: #ffffe0; -webkit-border-radius: 5px; -moz-border-radius: 5px; border-radius: 5px; margin: 0 5px;">
<strong>To fix this</strong>, log into your site via FTP and set the permissions <u>of the files and/or directories listed above</u> to 777
</p>
<p>Your site will have the following problems if the issues above are not addressed:</p>
<p><em>%s</em></p>
<p>I would love to work properly, but currently I can\'t.</p>
</div>', implode("\n", $failed), implode("<br/>", $effects));

    }
    
    if (!function_exists('imagecreatetruecolor')) {
      
      printf('<div  class="error" style="padding: 5px 5px;">
<h3 style="margin: 0.4em 0 0 0;">Thumbnails Issue</h3>
<p>It appears your server doesn\'t have the <strong><a target="_blank" href="http://php.net/manual/en/image.installation.php">PHP GD Library</a></strong> installed. The library is needed for the image cropping/resizing on your site. If the library is not installed,
you will not be able to see any thumbnails on your site. <strong>This is not a BUG</strong>.</p>
<p>You can ask your hosting provider to enable/install the library on your Server\'s PHP Configuration. For additional help, refer to this <a target="_blank" href="http://www.google.com/search?ie=UTF-8&q=php+enable+gd+library&oq=php+enable+gd+library">google search</a>.</p>
</div>');

    }
    
    if (!function_exists('mcrypt_encrypt')) {
      
      printf('<div  class="error" style="padding: 5px 5px;">
<h3 style="margin: 0.4em 0 0 0;">Contact Forms Issue</h3>
<p>Unable to find the <strong><a target="_blank" href="http://php.net/manual/en/function.mcrypt-encrypt.php">mcrypt_encrypt</a></strong> function. The function is used to encrypt your information when rendering contact forms using the Form Builder. This prevents
the disclosure of your email, both for privacy and to protect you from Spam Bots. <strong>This is not a BUG</strong>.</p>
<p>You can ask your hosting provider to enable the <strong><a target="_blank" href="http://www.php.net/manual/en/intro.mcrypt.php">MCrypt Extension</a></strong> on your Server\'s PHP Configuration. For additional help, refer to this <a target="_blank" href="https://www.google.com/search?ie=UTF-8&q=php+enable+mcrypt">google search</a>.</p>
</div>');

    }
    
  }
  
  
  /* Webfonts update callback */
  
  function webfonts_update_callback() {
    require(TEMPLATEPATH . '/framework/webfonts/update.php');
  }
  
  /* Theme Options update callback */
  
  function theme_options_update_callback() {
    require(TEMPLATEPATH . '/framework/theme-options/theme-options-action.php');
  }
  
  /* Sendmsg submit action */
  
  function sendmsg_submit() {
    require(TEMPLATEPATH . '/framework/mailer/sendmsg.php');
  }
  
  /* Theme Thickbox labels */
  
  function theme_thickbox_labels($translated, $text='', $domain='') {
    if ($translated == "Insert into Post") {
      if (isset($_GET['label'])) {
        switch ($_GET['label']) {
          case 'chooseThisImage':
            return "Choose This Image";
            break;
          default:
            return $translated;
            break;
        }
      } else {
        return $translated;
      }
    } else {
      return $translated;
    }
  }
  
  /* Theme Options Page */
  
  function theme_options_page() { global $der;
    $context = $_GET['page'];
    if ($context != 'layout-editor') {
      define('THEME_OPTIONS', true);
      $der->set_context($context);
      require($der->path('framework/classes/class.theme-options.php'));
      include_once($der->path(sprintf('includes/options/%s.php', $context)));
      new ThemeOptions();
      $der->reset_context();
    }
  }
  
  /* Register theme options page */
  
  function theme_register_options_page() { global $der, $theme_admin_menus;
    
    // Add theme options page
    __blocked_menu_page(null, $der->theme_data->get('Name'), THEME_CAPABILITY, 'theme-options', 'theme_options_page', $der->uri('core/images/admin/admin-menu-icon.png', THEME_VERSION), 56 );
    
    // Add submenu pages
    __blocked_submenu_page('theme-options', 'Theme Options', 'Theme Options', THEME_CAPABILITY, 'theme-options', 'theme_options_page');
    if (is_array($theme_admin_menus)) {
      foreach ($theme_admin_menus as $slug => $title) {
        __blocked_submenu_page('theme-options', $title, $title, THEME_CAPABILITY, $slug, 'theme_options_page');
      }
    }
    
    // Add color editor page
    __blocked_menu_page("Color Editor", "Colors", THEME_CAPABILITY, 'color-editor', 'theme_options_page', $der->uri('framework/assets/color-editor.png', THEME_VERSION), 58 );
    
  }
  
  /* Renders admin styles */

  function theme_admin_stylesheet() { global $der;
    wp_enqueue_style('framework-admin', $der->uri('framework/assets/css/admin.css'), array(), THEME_VERSION);
    wp_enqueue_style('framework-theme-options', $der->uri('framework/theme-options/theme-options.css'), array(), THEME_VERSION);
    wp_enqueue_style('framework-prettyphoto', $der->uri('framework/assets/prettyPhoto/css/prettyPhoto.css'), array(), THEME_VERSION);
    wp_enqueue_style('framework-prettyphoto', $der->uri('framework/assets/prettyPhoto/css/prettyPhoto.css'), array(), THEME_VERSION);
    wp_enqueue_style('framework-colorpicker', $der->uri('framework/assets/colorpicker/css/colorpicker.css'), array(), THEME_VERSION);
  }
  
  /* Renders the admin scripts */
  
  function theme_admin_print_scripts() { global $der;
    
    wp_enqueue_style('thickbox');
    wp_enqueue_script('media-upload');
    wp_enqueue_script('thickbox');
    
    ////////////////////////////////////////////////
    // TODO: Convert single quotes to double quotes
    ////////////////////////////////////////////////
    
    $update_server = (defined('THEME_UPDATE_SERVER')) ? THEME_UPDATE_SERVER : '';
    
    // Encode JSON, prevents users from breaking admin
    $tf_data = base64_encode(json_encode($der->options('tf_username, tf_purchase_code')));
    
    $theme = wp_get_theme(); // Note: Using compat layer (function introduced in 3.4.0)
    echo "
<script type='text/javascript'>
  document.WP_THEME_ID = '" . THEME_ID . "';
  document.WP_HOME_URL = '" . get_home_url() . "';
  document.WP_ADMIN_URL = '" . admin_url() . "';
  document.WP_THEME_NAME = '" . $theme->get('Name') . "';
  document.WP_THEME_VERSION = '" . $theme->get('Version') . "';
  document.WP_UPDATE_SERVER = '" . $update_server . "';
  document.WP_THEME_URI = '" . $der->uri('') . "';
  document.TF_DATA = '" . $tf_data . "';
  document.GI_THUMB_WIDTH = " . GI_THUMB_WIDTH . ";
  document.GI_THUMB_HEIGHT = " . GI_THUMB_HEIGHT . ";
  document.WP_I18N_COMMON = {
    newUpdateAvailable: '" . 'new update available' . "',
    documentation: '" . 'Documentation' . "',
    supportForums: '" . 'Support Forums' . "',
    getItNow: '" . 'Get it now!' . "',
    updateInstructions: '" . 'Update Instructions' . "',
    isAvailable: '" . 'is available' . "',
    themeOptions: '" . 'Theme Options' . "',
    layoutEditor: '" . 'Layout Editor' . "',
    savedSettings: '" . "Saved Settings" . "',
    resetSettings: '" . "Settings Reset" . "',
    resetSettingsMessage: '" . "Settings will be reset. Proceed ?" . "',
    uploadImage: '" . "Upload Image" . "',
    removeImage: '" . "Remove Image" . "',
    versionAvailable: '" . "Version %s available" . "',
    add: '" . "Add" . "',
    cancel: '" . "Cancel" . "',
    rename: '" . "Rename" . "',
    confirmRemove: '" . "About to remove \"%s\". Proceed ?" . "',
    splitVertically: '" . "Split Vertically" .  "',
    threeColumns: '" . "Three Columns" . "',
    confirmSlotRemove: '" . "The whole container will be removed. Proceed ?" . "'
  };
</script>\n";

    if (is_multisite()) { global $blog_id;

      echo '
<script type="text/javascript">
  document.WP_MULTISITE_ID = ' . $blog_id . ';
</script>' . "\n\n";

    }
    
  }
  
  /* Prints head scripts */
  
  function theme_admin_head() { global $der;
    wp_enqueue_script('framework-sanitizer', $der->uri('framework/assets/js/sanitizer.js'), array(), THEME_VERSION);
    wp_enqueue_script('framework-jwerty', $der->uri('framework/assets/js/jwerty.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('jquery-base64',	$der->uri('framework/assets/js/jquery.base-sixty-four.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('framework-hogan.js', $der->uri('framework/assets/js/hogan-2.0.0.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('framework-prettyphoto', $der->uri('framework/assets/prettyPhoto/jquery.prettyPhoto.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('framework-colorpicker', $der->uri('framework/assets/colorpicker/js/colorpicker.js'), array(), THEME_VERSION);
    wp_enqueue_script('framework-gallery-interface', $der->uri('framework/assets/js/gallery-interface.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('framework-admin', $der->uri('framework/assets/js/admin.js'), array('jquery'), THEME_VERSION);
    wp_enqueue_script('framework-theme-options', $der->uri('framework/theme-options/theme-options.js'), array('jquery'), THEME_VERSION);
  }
  
  /* Theme activation hook */
  
  function theme_activation_hook() {
    flush_rewrite_rules();
  }
  
  /* Returns the taxonomy links used on post listing columns */

  function theme_list_taxonomies_column($post_id, $taxonomy) {
    $genres = wp_get_post_terms($post_id, $taxonomy);
    $post_type = get_post_type($post_id);
    $links = array();
    foreach ($genres as $term) {
      $term_name = $term->name;
      $term_slug = $term->slug;
      $links[] = sprintf('<a href="%s">%s</a>', admin_url("edit.php?post_type=${post_type}&taxonomy=${taxonomy}&${taxonomy}=${term_slug}"), $term_name);
    }
    echo implode(', ', $links);
  }
  
  /* Returns taxonomy data using slug => title */
  
  function theme_get_taxonomy_data($taxonomy) {
    $out = array();
    $terms = get_terms($taxonomy, array('hide_empty'=>false));
    foreach ($terms as $tax) {
      $out[$tax->slug] = esc_html($tax->name);
    }
    return $out;
  }
  
?>