<?php

  add_shortcode('accordion', 'accordion_shortcode');
  add_shortcode('tabs', 'tabs_shortcode');
  add_shortcode('tab', 'tab_shortcode');
  add_shortcode('toggle', 'toggle_shortcode');
  add_shortcode('ibox', 'ibox_shortcode');
  add_shortcode('push', 'push_shortcode');
  add_shortcode('pull', 'pull_shortcode');
  add_shortcode('clear', 'clear_shortcode');
  add_shortcode('testimonial', 'testimonial_shortcode');
  add_shortcode('slogan', 'slogan_shortcode');
  add_shortcode('blockquote', 'blockquote_shortcode');
  add_shortcode('capsule', 'capsule_shortcode');
  add_shortcode('button', 'button_shortcode');
  add_shortcode('audioplayer', 'audio_player_shortcode');
  
  function audio_player_shortcode($atts, $content='', $code='') {
    
    $defaults = array(
      'file' => null,
    );
    
    extract(shortcode_atts($defaults, $atts));
    
    return sprintf('<div class="multimedia media-type-audio" style="margin-top: 1.5em;"><audio src="%s"></audio></div>', $file);

  }
  
  function button_shortcode($atts, $content='', $code='') {
    
    $defaults = array(
      'external' => false,
      'href' => null,
      'type' => null, // solid, big
      'full_width' => false
    );
    
    extract(shortcode_atts($defaults, $atts));
    
    $btn_class = ($type == 'solid') ? 'solid-button' : 'button';
    
    if ($type == 'big') $btn_class .= ' big';
    
    if ($full_width) $btn_class .= ' full-width text-center';
    
    $out = sprintf('<a class="%s"', $btn_class);
    
    if ($external) $out .= ' target="_blank"';
    
    if ($href) $out .= sprintf(' href="%s"', $href);
    
    $out .= '>' . esc_html($content) . '</a>';

    return $out;
    
  }
  
  function capsule_shortcode($atts, $content='', $code='') {
    
    $defaults = array(
      'external' => false,
      'href' => null
    );
    
    extract(shortcode_atts($defaults, $atts));
    
    $out = '<span class="capsule"><a';
    
    if ($external) $out .= ' target="_blank"';
    
    if ($href) $out .= sprintf(' href="%s"', $href);
    
    $out .= '>' . esc_html($content) . '</a></span>';
    
    return $out;
    
  }
  
  function blockquote_shortcode($atts, $content='', $code='') { global $der;
    
    $defaults = array(
      'align' => 'none',
      'width' => 200
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    extract($args);
    
    $content = $der->content($content);
    
    $width = ($align == 'center') ? 'auto' : $width . 'px';
    
    return sprintf('<blockquote class="special align%s" style="width: %s;">
%s    
</blockquote>', $align, $width, $content);
    
  }
  
  function slogan_shortcode($atts, $content='', $code='') { global $der;
    
    $defaults = array(
      'bg' => false,
      'action' => null,
      'url' => null,
      'solid' => false
    );
    
    $args = wp_parse_args($atts, $defaults);

    extract($args);
    
    $class = $bg ? 'slogan bg' : 'slogan';
    
    $content = remove_br($content);
    
    if ($action) {
      
      $class .= ' cta';
      
      $btn_class = (strtolower($solid) == 'true') ? 'solid-button' : 'button';
      
      $content .= sprintf(' <a class="%s float-right" href="%s">%s</a>', $btn_class, $url, $action);
      
    }
    
    return sprintf("\n".'<p class="%s">%s</p>', $class, $content);
    
  }

  function testimonial_shortcode($atts, $content='', $code='') { global $der;
    
    $defaults = array(
      'from' => '',
      'url' => ''
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    $args['content'] = trim(remove_br($content));
    
    return $der->render('
<div class="testimonial">
  <blockquote>
    {{{content}}}
  </blockquote>
  {{#from}}{{#url}}<a href="{{{url}}}">&ndash; {{from}}</a>{{/url}}{{^url}}<strong>&ndash; {{from}}</strong>{{/url}}{{/from}}
</div><!-- .quote -->', $args);
    
  }
  
  function clear_shortcode($atts, $content='', $code='') {
    
    // Note: height and push have the same effect.
    // push is an alias of height
    
    $defaults = array(
      'height' => 0,
      'push' => 0
    );

    $args = wp_parse_args($atts, $defaults);
    
    extract($args);
    
    $height = $height ? $height : $push;
    
    if ($height) {
      return sprintf("\n".'<div class="clear push-%d"></div>', $height);
    } else {
      return sprintf("\n".'<div class="clear"></div>');
    }
    
  }
  
  function push_shortcode($atts, $content='', $code='') {
    
    $defaults = array(
      'height' => 30,
      'clear' => false,
      'separator' => false
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    extract($args);
    
    $class = ($clear) ? 'clear' : '';
    $class .= sprintf(' push push-%d', $height);
    
    if ($separator) $class.= ' separator';
    
    return sprintf("\n".'<div class="%s"></div>', $class);
    
  }
  
  function pull_shortcode($atts, $content='', $code='') {
    
    $defaults = array(
      'height' => 10,
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    extract($args);
    
    return sprintf("\n".'<div class="pull-%d"></div>', $height);
    
  }

  function tabs_shortcode($atts, $content='', $code='') { global $der;
    
    // Shortcode accepts no arguments
    
    $der->tabs_context = 'tabs';
    
    $args = array();
    $args['content'] = $der->content($content);
    
    return $der->render('
<ul class="tabs">
{{{content}}}
</ul><!-- .tabs -->', $args);
    
  }

  function ibox_shortcode($atts, $content='', $code='') { global $der;

    $defaults = array(
      'type' => 'notice',
      'collapse' => false
    );

    $args = wp_parse_args($atts, $defaults);

    $args['content'] = trim(remove_br($content));

    return $der->render('
<div class="ibox {{{type}}}{{#collapse}} collapse{{/collapse}}">
{{{content}}}
</div><!-- ibox.{{{type}}} -->', $args);

  }
  
  function toggle_shortcode($atts, $content='', $code='') { global $der;

    $defaults = array(
      'title' => '',
      'active' => false
    );

    $args = wp_parse_args($atts, $defaults);

    $args['title'] = apply_filters('the_title', $args['title']);
    $args['content'] = $der->content($content);

    return $der->render('
<div class="toggle-box ui-toggle{{#active}} active{{/active}}">
  <h3 class="title">{{{title}}}</h3>
  <div class="content">
    {{{content}}}
  </div><!-- .content -->
</div><!-- .toggle-box -->', $args);

  }
  
  function accordion_shortcode($atts, $content='', $code='') { global $der;
    
    $der->tabs_context = 'accordion';
    
    $defaults = array(
      'type' => null  // Acceptable types: slim, separator
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    $args['content'] = $der->content($content);

    switch ($args['type']) {
      case 'separators':
        $args['class'] = 'slim separators';
        break;
      default:
        $args['class'] = $args['type'];
    }
    
    return $der->render('
<ul class="accordion-boxes{{#class}} {{{class}}}{{/class}}">
{{{content}}}
</ul><!-- .accordion-boxes -->', $args);
    
  }
  
  function tab_shortcode($atts, $content='', $code='') { global $der;
    
    $defaults = array(
      'title' => '',
      'active' => false
    );
    
    $args = wp_parse_args($atts, $defaults);
    
    $args['title'] = apply_filters('the_title', $args['title']);
    $args['content'] = $der->content($content);
    
    $context = $der->tabs_context;
    
    if ($context == 'accordion') {
      
      return $der->render('
<li class="unit ui-toggle{{#active}} active{{/active}}">
<h3 class="title">{{{title}}}</h3>
<div class="content">
  {{{content}}}
</div><!-- .content -->
</li><!-- .unit -->', $args);
      
    } else if ($context == 'tabs') {
      
      return $der->render('
<li class="tab{{#active}} active{{/active}}">
  <h3 class="title">{{{title}}}</h3>
  {{{content}}}
</li><!-- .tab -->', $args);
      
    }
    
  }
  
?>