<?php

add_action('widgets_init', 'AdvertisingWidget::register');

class AdvertisingWidget extends WidgetFramework {
  
  function __construct() {
    
    $this->initialize(array(
      'id' => 'advertising-component',
      'title' => "Advertising",
      'description' => "Displays advertising links",
      'gallery_interface' => 'content',
      'defaults' => array(
        'title' => __("Advertising", "theme"),
        'gallery' => null,
        'cols' => 'two-cols',
        'width' => 200,
        'height' => 200,
        'target' => 'default'
      )
    ));

  }
  
  static function register() {
    register_widget(__CLASS__);
  }
  
  function render($args) { global $der;
    unset($args['title']);
    $args['gallery'] = preg_replace('/(^__|__$)/', '', $args['gallery']);
    $args['is_widget'] = true;
    $args['target'] = ($args['target'] == '_blank');

    extract($args);

    if (is_numeric($gallery)) {
      $images = json_decode(get_post_meta($gallery, $der->key('gallery_images'), true), true);
      if (!is_array($images)) $images = array();
    } else {
      $images = array();
    }

    foreach ($images as $i => $data) {
      $images[$i]['image'] = $der->thumb_src($data['image'], $width, $height);
      $images[$i]['title'] = esc_html($data['title']);
      if ($data['description']) $images[$i]['title'] .= ' &ndash; ' . esc_html($data['description']);
    }

    $args['images'] = $images;

    echo $der->render('
<ul class="resp-gallery {{{cols}}} clearfix">
{{#images}}
  <li><a {{#target}} target="_blank"{{/target}} title="{{{title}}}" class="frame no-border"{{#url}} href="{{{url}}}"{{/url}}><img alt="{{{title}}}" src="{{{image}}}" /></a></li>
{{/images}}
</ul><!-- .resp-gallery -->', $args);
    
  }
  
  function admin($instance) {
    
    $this->gallery_select(array(
      'id' => 'gallery',
      'title' => 'Gallery',
      'description' => "Gallery to choose images from."
    ));
    
    $this->text(array(
      'id' => 'width',
      'title' => "Thumb Width (pixels)",
      'description' => "Width to use for the image."
    ));
    
    $this->text(array(
      'id' => 'height',
      'title' => "Thumb Height (pixels)",
      'description' => "Height to use for the image."
    ));
    
    $this->select(array(
      'id' => 'cols',
      'title' => "Columns",
      'description' => "Column Distribution to use",
      'values' => array(
        'one-col' => "One Column",
        'two-cols' => "Two Columns",
        'three-cols' => "Three Columns",
        'four-cols' => "Four Columns"
      )
    ));
    
    $this->select(array(
      'id' => 'target',
      'title' => "Links Open Behavior",
      'description' => "Behavior when opening links.",
      'values' => array(
        'default' => "Same Window",
        '_blank' => "External Window"
      )
    ));
    
  }
  
}

?>