<?php

add_action('widgets_init', 'LatestPostsWidget::register');

class LatestPostsWidget extends WidgetFramework {
  
  function __construct() {
    
    $this->initialize(array(
      'id' => 'latest-posts-component',
      'title' => "Latest Posts",
      'description' => "Displays your latest posts",
      'update_taxonomies' => array('category'),
      'defaults' => array(
        'title' => __("Latest Posts", "theme"),
        'showposts' => 5,
        'thumb_width' => 63,
        'thumb_height' => 50,
        'category' => null,
        'metadata_display' => 'raw-date',
        'show_comments' => 'yes'
      )
    ));

  }
  
  static function register() {
    register_widget(__CLASS__);
  }
  
  /* Render the widget form */
  
  function render($instance) { global $der;
    $instance['show_comments'] = str2bool($instance['show_comments']);
    $instance['category'] = $this->get_checked_taxonomies('category');
    $instance['is_widget'] = true;
    echo latest_posts_component($instance);
  }
  
  function admin($instance) {
    
    $this->text(array(
      'id' => 'showposts',
      'title' => "Posts to Display",
      'description' => 'Number of posts to display in the widget.'
    ));

    $this->taxonomy_checkbox(array(
      'taxonomy' => 'category',
      'title' => 'Categories',
      'description' => 'Categories to retrieve posts from. If left blank, all categories are used.'
    ));
    
    $this->text(array(
      'id' => 'thumb_width',
      'title' => "Thumb Width (pixels)",
      'description' => 'Width to use for the thumbnails.'
    ));

    $this->text(array(
      'id' => 'thumb_height',
      'title' => "Thumb Height (pixels)",
      'description' => 'Height to use for the thumbnails.'
    ));
    
    $this->select(array(
      'id' => 'metadata_display',
      'title' => 'Show as Description',
      'description' => 'Data to display in the description area.',
      'values' => array(
        'description' => "Post Description",
        'author-date' => "Author / Date",
        'date' => "Published Date",
        'raw-date' => "Date",
        'cats' => "Categories",
        'tags' => "Tags",
        'cats-tags' => "Categories / Tags",
        'author-date-cats' => "Author / Date / Categories",
        'author-date-cats-tags' => "Author / Date / Categories / Tags",
        'author-cats' => "Author / Categories",
        'author-cats-tags' => "Author / Categories / Tags" 
      )
    ));
    
    $this->select(array(
      'id' => 'show_comments',
      'title' => 'Show Comments',
      'description' => 'Whether or not to show comments.',
      'values' => array(
        'yes' => "Yes",
        'no' => "No"
      )
    ));
    
  }

}

function latest_posts_component($atts) { global $der;
  
  $defaults = array(
    'section_title' => '',
    'showposts' => 5,
    'thumb_width' => 63,
    'thumb_height' => 50,
    'category' => null,
    'metadata_display' => 'raw-date',
    'show_comments' => true,
    'is_widget' => false
  );
  
  $args = wp_parse_args($atts, $defaults);
  
  $args = component_get_posts($args, array(
    'width' => $args['thumb_width'],
    'height' => $args['thumb_height'],
    'clearing' => false,
    'excerpt' => false,
    'show_image' => true,
    'show_description' => true,
    'post_class' => false,
    'metadata_display' => 'raw-date',
    'resize_image' => false,
    'get_comments_open' => true,
    'get_comment_number' => true
  ));
  
  return $der->render('
{{^is_widget}}<div class="latest-posts-component component {{{columns_class}}}{{#hide_on_mobile}} mobile-hide{{/hide_on_mobile}}">
{{#section_title}}<h3 class="section-title">{{section_title}}</h3>{{/section_title}}
{{/is_widget}}
<ul>
{{#posts}}
<li>
  <a class="bordered" href="{{{permalink}}}"><img alt="{{{title}}}" width="{{{thumb_width}}}" height="{{{thumb_height}}}" src="{{{image}}}" /></a>
  <aside>
    <a class="title" href="{{{permalink}}}">{{{title}}}</a>
    <small class="desc">{{{description}}}</small>
    {{#show_comments}}{{#comments_open}}<small class="desc">{{comment_number}}</small>{{/comments_open}}{{/show_comments}}
  </aside>
</li>
{{/posts}}
</ul>
{{^is_widget}}</div><!-- .latest-posts-component -->{{/is_widget}}', $args);
  
}

?>